"""
This is a setup.py script generated by py2applet

Usage:
    python setup.py py2app
"""
import py2app
import periodictable
import numpy as np
from setuptools import setup, Extension, find_packages

try:
    from Cython.Distutils import build_ext
except ImportError:
    USE_CYTHON = False
else:
    USE_CYTHON = True

# Obtain the numpy include directory.  This logic works across numpy versions.
try:
    numpy_include = np.get_include()
except AttributeError:
    numpy_include = np.get_numpy_include()

ext_modules = []

if USE_CYTHON:
    # creflect extension module
    _creflect = Extension(
                          name = 'refnx.analysis._creflect',
                          sources=['src/_creflect.pyx', 'src/refcalc.cpp'],
                          include_dirs = [numpy_include],
                          language = 'c',
                          extra_link_args = ['-lpthread']
                          # libraries=
                          # extra_compile_args = "...".split(),
                          )
    ext_modules.append(_creflect)

APP = ['motofit/motofit.py']
DATA_FILES = []
PACKAGES = ['refnx',
            'periodictable',
            'lmfit',
            'pymc']

VERSION = '0.0.1'

CFBUNDLEDOCUMENTYPES = [{'CFBundleTypeIconFiles': 'Motofit.icns',
                         'CFBundleTypeExtensions': ['mtft']}]

OPTIONS = {'argv_emulation': True,
           'includes': ['scipy', 'numpy', 'matplotlib', 'lmfit', 'refnx'],
           'packages': PACKAGES,
           'iconfile': 'motofit/icons/Motofit.icns',
           'excludes': ['refnx.reduce'],
           'plist': dict(CFBundleIconFile='Motofit.icns',
                         CFBundleDocumentTypes=CFBUNDLEDOCUMENTYPES,
                         CFBundleShortVersionString=VERSION,
                         CFBundleVersion=VERSION)}

setup(
    name='Motofit',
    app=APP,
    data_files=DATA_FILES,
    ext_modules=ext_modules,
    cmdclass = {'build_ext': build_ext},
    options={'py2app': OPTIONS},
    setup_requires=['py2app'],
)
